{ ************************************************************************** }
{                                                                            }
{   VCL object TJPMColorListObject class                                     }
{                                                                            }
{:   This is just an object for handling tasks with Color Selection controls }
{                                                                            }
{   Copyright  2000-2003 by J. Peter Mugaas                                      }
{                                                                            }
{ ************************************************************************** }

{Revision notes:

1-29-2000 started this interface
1-30-2000 code adjustments had to be made for ShowBoxWidth
1-31-2000 fixed a bug where the color was not preserved before calling inherited draw
    This effected non-ESBPCS related ComboBoxes
6-June-2000 ESB Consultancy added ESBReadOnlyColor
}
unit JPMColorListObject;

interface

{$I JPM.INC}

uses Classes, Dialogs, Graphics, JPMGenListObject,
     {$IFDEF BelowD5}StdCtrls, {$ENDIF}
     Windows;
{: This unit contains the TJPMColorListObject which is a color selection object
 for color selection controls such as a listbox or combobox.
}
type
     TJPMColorListObject = class (TJPMGenListObject)
     private
          FSelectColor: TColor;
          FShowAdditionalColors: Boolean;
          FShowSystemColors: Boolean;
          FColorSelDialog: TColorDialog;
          FShowCustomColor: Boolean;
          FTextOnColorBackground: Boolean;
          function GetSelectColor: TColor;
          {: Write method for property SelectColor }
          procedure SetSelectColor (Value: TColor);
          {: Read method for property ShowSystemColors }
          function GetShowSystemColors: Boolean;
          {: Write method for property AdditionalColors}
          procedure SetShowAdditionalColors (Value: Boolean);
          {: Write method for property ShowSystemColors }
          procedure SetShowSystemColors (Value: Boolean);
          {: Write method for property ShowCustomColor }
          procedure SetShowCustomColor (Value: Boolean);
          function GetTextOnColorBackground: Boolean;
          procedure SetTextOnColorBackground (const Value: Boolean);
     protected
          {: Set draw style for the text not in the preview box}
          procedure CanvasDrawStyle (const ACanvas: TCanvas; AIndex: Integer); override;

          {: Method to do any intialization}
          procedure AutoInitialize; override;
          {: Method to free any objects created by AutoInitialize }
          procedure AutoDestroy; override;
     public
          {:This is used in descendants for populating the list control, this one just clears the Items}
          procedure PopulateComboBox; override;
          {: In descendants, this will draw a box in DrawItem overrides, this abstract version just sets Rect.Left to Rect.Left + 40 if ShowBox}
          procedure DrawBox (Canvas: TCanvas; Index: Integer; var Rect: TRect; State: TOwnerDrawState); override;

          {: Execute the color selection dialog-box}
          function ExecDialogBox: Boolean;
          {: Color which was selected in the combo-box }
          function ExecDialogBoxEnh (CustomColors: TStrings): Boolean;
          {: Color which was selected in the combo-box }
          property SelectColor: TColor read GetSelectColor write SetSelectColor;
          {: Show system colors such as clWindows }
          property ShowSystemColors: Boolean read GetShowSystemColors write SetShowSystemColors;
          {: Show additional colors such as Pink, Lemon, etc.}
          property ShowAdditionalColors: Boolean read FShowAdditionalColors write SetShowAdditionalColors;
          {: Show the Custom Color item which pops-up the color selection dialog}
          property ShowCustomColor: Boolean read FShowCustomColor write SetShowCustomColor;
          {: Show the text on using the color as a background}
          property TextOnColorBackground: Boolean read GetTextOnColorBackground write SetTextOnColorBackground;

     end;

var
     {: Stores the Custom Colors that are displayed in the Dialog }
     JPMCustomColors: TStrings;

implementation
uses JPMResourceStrings;

{addional colors metioned by Glenn Crouch which are included with Reportprinter
Pro and he advised me that "Jim Gunkel, Nevronal said we were free to use them"  }
const
     clPink = $8080FF;
     clRedClay = $404080;
     clNightRed = $000040;
     clLemon = $80FFFF;
     clPeach = $4080FF;
     clOrange = $0080FF;
     clBrown = $004080;
     clMistGreen = $80FF80;
     clYellowGreen = $00FF80;
     clNightGreen = $004000;
     clGold = $408080;
     clSeaGreen = $80FF00;
     clGreenApple = $40FF00;
     clBlueGreen = $408000;
     clForestGreen = $404000;
     clSkyBlue2 = $FFFF80;
     clDeepBlue = $804000;
     clTealGray = $808040;
     clGreenBlue = $FF8000;
     clTealBlue = $C08000;
     clLavender = $FF8080;
     clRoyalBlue = $A00000;
     clNightBlue = $400000;
     clBubbleGum = $C080FF;
     clPeriwinkle = $C08080;
     clPlum = $400080;
     clBlackCherry = $400040;
     clPinkCarnation = $FF80FF;
     clVioletRed = $8000FF;
     clLightPurple = $FF0080;
     clDarkPurple = $800040;

     { Additional Colors from ESBPCS }
const
     clSteel = $CCBDB5;
     clPaleBlue = $FFFFF0;
     clTickGreen = $0021A121;

     {$IFDEF BelowD6}
     {Allowing for extra System Colors introduced in D6 }
const
     clHotLight = clHighlight;
     clGradientActiveCaption = clActiveCaption;
     clGradientInactiveCaption = clInactiveCaption;
     clMenuHighlight = clHighlight;
     clMenuBar = clBtnFace;

     {Allowing for Delphi 6 Extra Colors}
const
     clMoneyGreen = $C0DCC0;
     clSkyBlue = $F0CAA6;
     clCream = $F0FBFF;
     clMedGray = $A4A0A0;
     {$ENDIF}

     // these constants are for color names and the colors they refer to.
const
     MAX_REG_COLORS = 19; //last color in Fixed Colors array
     MAX_ADD_COLORS = 33; //last color in Additional Colors array
     MAX_SYS_COLORS = 31; //last color in System Colors array

const
     RegularColors: array [0..MAX_REG_COLORS] of TColor = (
          clBlack,
          clMaroon,
          clGreen,
          clOlive,
          clNavy,
          clPurple,
          clTeal,
          clGray,
          clSilver,
          clred,
          clLime,
          clYellow,
          clBlue,
          clFuchsia,
          clAqua,
          clWhite,
          clMoneyGreen,
          clSkyBlue,
          clCream,
          clMedGray);
     RegularColorText: array [0..MAX_REG_COLORS] of string = (
          RSJBLACK,
          RSJMAROON,
          RSJGREEN,
          RSJOLIVEGREEN,
          RSJNAVYBLUE,
          RSJPURPLE,
          RSJTEAL,
          RSJGRAY,
          RSJSILVER,
          RSJRED,
          RSJLIMEGREEN,
          RSJYELLOW,
          RSJBLUE,
          RSJFUCHSIA,
          RSJAQUA,
          RSJWHITE,
          RSJMONEYGREEN,
          RSJSKYBLUE,
          RSJCREAM,
          RSJMEDGRAY);

     AdditionalColors: array [0..MAX_ADD_COLORS] of TColor = (
          clPink,
          clRedClay,
          clNightRed,
          clLemon,
          clPeach,
          clOrange,
          clBrown,
          clMistGreen,
          clYellowGreen,
          clNightGreen,
          clGold,
          clTickGreen,
          clSeaGreen,
          clGreenApple,
          clBlueGreen,
          clForestGreen,
          clPaleBlue,
          clSkyBlue2,
          clDeepBlue,
          clTealGray,
          clGreenBlue,
          clTealBlue,
          clLavender,
          clRoyalBlue,
          clNightBlue,
          clBubbleGum,
          clPeriwinkle,
          clPlum,
          clBlackCherry,
          clPinkCarnation,
          clVioletRed,
          clLightPurple,
          clDarkPurple,
          clSteel);
     AdditionalColorText: array [0..MAX_ADD_COLORS] of string = (
          RSJPINK,
          RSJREDCLAY,
          RSJNIGHTRED,
          RSJLEMON,
          RSJPEACH,
          RSJORANGE,
          RSJBROWN,
          RSJMISTGREEN,
          RSJYELLOGREEN,
          RSJNIGHTGREEN,
          RSJGOLD,
          RSJTICKGREEN,
          RSJSEAGREEN,
          RSJGREENAPPLE,
          RSJBLUEGREEN,
          RSJFORESTGREEN,
          RSJPALEBLUE,
          RSJSKYBLUE2,
          RSJDEEPBLUE,
          RSJTEALGRAY,
          RSJGREENBLUE,
          RSJTEALBLUE,
          RSJLAVENDAR,
          RSJROYALBLUE,
          RSJNIGHTBLUE,
          RSJBUBBLEGUM,
          RSJPERIWINKLE,
          RSJPLUM,
          RSJBLACKCHERRY,
          RSJPINKCARNATION,
          RSJVIOLETRED,
          RSJLIGHTPURPLE,
          RSJDARKPURPLE,
          RSJSTEEL);

     SystemColors: array [0..MAX_SYS_COLORS] of TColor = (clBackground,
          clActiveCaption,
          clInactiveCaption,
          clGradientActiveCaption,
          clGradientInactiveCaption,
          clCaptiontext,
          clInactiveCaptiontext,
          clMenu,
          clMenuText,
          clMenuHighlight,
          clMenuBar,
          clWindow,
          clWindowText,
          clWindowFrame,
          clActiveBorder,
          clInactiveBorder,
          clAppWorkSpace,
          clHighlight,
          clHighlightText,
          clBtnFace,
          clBtnShadow,
          clBtnHighlight,
          clBtnText,
          clGrayText,
          cl3DDkShadow,
          cl3DLight,
          clInfoText,
          clInfoBk,
          clScrollbar,
          clHotLight,
          clDefault,
          clNone);
     SystemColorText: array [0..MAX_SYS_COLORS] of string = (RSJBACKGROUND,
          RSJACTIVECAPTION,
          RSJINACTIVECAPTION,
          RSJGRADIENTACTIVECAPTION,
          RSJGRADIENTINACTIVECAPTION,
          RSJCAPTIONTEXT,
          RSJINACTIVECAPTIONTEXT,
          RSJMENU,
          RSJMENUTEXT,
          RSJMENUHIGHLIGHT,
          RSJMENUBAR,
          RSJWINDOWBACKGROUND,
          RSJWINDOWTEXT,
          RSJWINDOWFRAME,
          RSJACTIVEBORDER,
          RSJINACTIVEBORDER,
          RSJAPPLICATIONWORKSPACE,
          RSJSELECTIONBACKGROUND,
          RSJSELECTIONTEXT,
          RSJBUTTONFACE,
          RSJBUTTONSHADOW,
          RSJBTNHIGHLIGHT,
          RSJBUTTONTEXT,
          RSJDIMMEDTEXT,
          RSJ3DDARKSHADOW,
          RSJ3DLIGHT,
          RSJTOOLTIPTEXT,
          RSJTOOLTIPBACKGROUND,
          RSJSCROLLBAR,
          RSJHOTLIGHT,
          RSJDEFAULT,
          RSJNONE);
     CustomColor: string = RSJCUSTOMCOLOR;
     { TJPMColorListObject }

procedure TJPMColorListObject.AutoDestroy;
begin
     inherited;
     FColorSelDialog.Free;
end;

procedure TJPMColorListObject.AutoInitialize;
begin
     inherited;
     FColorSelDialog := TColorDialog.Create (nil);
     FColorSelDialog.Options := [cdFullOpen, cdSolidColor, cdAnyColor];
     FTextOnColorBackground := False;
end;

procedure TJPMColorListObject.CanvasDrawStyle (const ACanvas: TCanvas;
     AIndex: Integer);
begin
     if FShowStyledText then
     begin
          ACanvas.Font.Color := TColor (FItems.Objects [AIndex]);
     end;
end;

procedure TJPMColorListObject.DrawBox (Canvas: TCanvas; Index: Integer;
     var Rect: TRect; State: TOwnerDrawState);
var
     SaveColor: TColor;
begin
     SaveColor := Canvas.Brush.Color; //save original color
     Canvas.Brush.Color := TColor (FItems.Objects [Index]);
     {Background color if requested}
     if FTextOnColorBackground then
     begin
          Canvas.FillRect (Rect);
     end;
     {Color preview rectange}
     DrawBoxOutLine (Canvas, Index, Rect, State);
     if ShowBox then
     begin
          if ShowBoxBorder then
               Canvas.FillRect (Classes.Rect (Rect.Left + 2, Rect.Top + 2, FShowBoxWidth - 1, Rect.Bottom - 2))
          else
               Canvas.FillRect (Classes.Rect (Rect.Left + 1, Rect.Top + 1, FShowBoxWidth - 1, Rect.Bottom - 1));
     end;
     if not FTextOnColorBackground then
          Canvas.Brush.Color := SaveColor; //restore original color
     {only call the inherited procedure for the part after the colors}
     inherited
end;

//this loads the colors into the listbox

function TJPMColorListObject.ExecDialogBox: Boolean;
begin
     Result := False; {assume false for expediancy}
     if FShowCustomColor then
     begin
          FColorSelDialog.Color := FSelectColor;
          if FColorSelDialog.Execute then
          begin
               FSelectColor := FColorSelDialog.Color; //indicate selected for our property
               {We add the value to the last item so that it is reflected in the combo-box color}
               FItems.Objects [FItems.Count - 1] := TObject (FSelectColor);
               Result := True;
          end; //if not ReadOnly and FColorSelDialog.Execute then
     end; //if FShowCustomColor and (ItemIndex = (Items.Count -1)) then
end;

//this loads the colors into the listbox

function TJPMColorListObject.ExecDialogBoxEnh (CustomColors: TStrings): Boolean; //esb
begin
     Result := False; {assume false for expediancy}
     if FShowCustomColor then
     begin
          FColorSelDialog.CustomColors.Assign (CustomColors);
          FColorSelDialog.Color := FSelectColor;
          if FColorSelDialog.Execute then
          begin
               FSelectColor := FColorSelDialog.Color; //indicate selected for our property
               {We add the value to the last item so that it is reflected in the combo-box color}
               FItems.Objects [FItems.Count - 1] := TObject (FSelectColor);
               CustomColors.Assign (FColorSelDialog.CustomColors);
               Result := True;
          end; //if not ReadOnly and FColorSelDialog.Execute then
     end; //if FShowCustomColor and (ItemIndex = (Items.Count -1)) then
end;

function TJPMColorListObject.GetSelectColor: TColor;
begin
     Result := FSelectColor;
end;

function TJPMColorListObject.GetShowSystemColors: Boolean;
begin
     Result := FShowSystemColors;
end;

function TJPMColorListObject.GetTextOnColorBackground: Boolean;
begin
     Result := FTextOnColorBackground;
end;

procedure TJPMColorListObject.PopulateComboBox;
var
     iIterator: Integer;
begin
     inherited PopulateComboBox;
     {add fixed-colors to combo-box}
     for iIterator := 0 to MAX_REG_COLORS do
     begin
          FItems.AddObject (RegularColorText [iIterator], Tobject (RegularColors [iIterator])); //we type case the TColor as an object for AddObject
     end; //for iIterator := 0 to MAX_REG_COLORS do

     if FShowAdditionalColors then
          {add additional colors to combo-box}
          for iIterator := 0 to MAX_ADD_COLORS do
          begin
               FItems.AddObject (AdditionalColorText [iIterator], Tobject (AdditionalColors [iIterator])); //we type case the TColor as an object for AddObject
          end; //for iIterator := 0 to MAX_ADD_COLORS do

     if FShowSystemColors then
          {add fixed-colors to combo-box}
          for iIterator := 0 to MAX_SYS_COLORS do
          begin
               FItems.AddObject (SystemColorText [iIterator], Tobject (SystemColors [iIterator])); //we type case the TColor as an object for AddObject
          end; //for iIterator := 0 to MAX_SYS_COLORS do

     if FShowCustomColor then
     begin
          FItems.Add (CustomColor);
          {If the color is not predifined, we will assume that it is a user's custom color}
          if FItems.IndexOfObject (TObject (FSelectColor)) = -1 then
               FItems.Objects [FItems.Count - 1] := TObject (FSelectColor);
     end; //if FShowCustomColor then
end;

procedure TJPMColorListObject.SetSelectColor (Value: TColor);
begin
     FSelectColor := Value;
end;

procedure TJPMColorListObject.SetShowAdditionalColors (Value: Boolean);
begin
     FShowAdditionalColors := Value;
end;

procedure TJPMColorListObject.SetShowCustomColor (Value: Boolean);
begin
     FShowCustomColor := Value;
end;

procedure TJPMColorListObject.SetShowSystemColors (Value: Boolean);
begin
     FShowSystemColors := Value;
end;

procedure TJPMColorListObject.SetTextOnColorBackground (
     const Value: Boolean);
begin
     FTextOnColorBackground := Value;
end;

initialization //esb

     JPMCustomColors := TStringList.Create

finalization //esb

     JPMCustomColors.Free;

end.
