{:This unit is a Font Collection list for use with the JPM Open Components and
 provides some Font listing capability even in console applications.<p>
 Under NO circumstances should forms be linked to this unit}
unit JPMFontList;
interface

{$I JPM.INC}

uses classes, Windows;

type
     {: This is a Font pitch - fpFixed means a fixed pitch such as a terminal Font.
     fpVariable is for a variable-width Font. fpDefault for Fonts which do not
     fit into these catagories or where the pitch is not known}
     TjpmFontPitch = (fpFixed, fpVariable, fpDefault);
     {: This set indicates the Font-pitches which should be listed in various controls}
     TjpmFontPitches = set of TjpmFontPitch;
     {: This type indicates a Font-family}
     TjpmFontFamily = (ffDecorative, ffUnknown, ffModern, ffRoman, ffScript, ffSwiss);
     {: This set contains the TjpmFontFamily.  In the future, it is hoped that this will be used in the controls}
     TjpmFontFamilies = set of TjpmFontFamily;
     {: This type indicates a type of Font such as Device, Raster, TrueType, or Other}
     TjpmFontype = (ftDevice, ftRaster, ftTrueType, ftOther);
     {: This set indicates Font-types which is used as a filter in various listing controls}
     TjpmFontypes = set of TjpmFontype;
     TJFontItem = class (TCollectionItem)
     private
          FFontTypeInt: Integer;
          FEnumLogFontEx: TEnumLogFontExA;
          FTextMetric: TTextMetric;
          function GetInternalFontPitch: Byte;
          function GetFontName: string;
          function GetFontPitch: TjpmFontPitch;
          function GetFamily: TjpmFontFamily;
          function GetFontType: TjpmFontype;
     public
          {: This is the Win32 API EnumLogFontExA for a Font.  We use it to extract other information about the Font}
          property EnumLogFontEx: TEnumLogFontExA read FEnumLogFontEx write FEnumLogFontEx;
          {: This is the Win32 API TextMetric for a Font.}
          property TextMetric: TTextMetric read FTextMetric write FTextMetric;
          {: This is a the Font type as an Integer.  This is used returned in the Win32 API EnumFontFamiliesEx function}
          property FontTypeInt: Integer read FFontTypeInt write FFontTypeInt;
          {: This is the Font type for this Font}
          property FontType: TjpmFontype read GetFontType;
          {: This is the Font's name}
          property FontName: string read GetFontName;
          {: This is the Font's pitch}
          property FontPitch: TjpmFontPitch read GetFontPitch;
          {: This is the Font's family}
          property FontFamily: TjpmFontFamily read GetFamily;
     end;
     TJPMFontList = class (TCollection)
     protected
          function GetItem (Index: Integer): TJFontItem;
          procedure SetItem (Index: Integer; const Value: TJFontItem);
     public
          {: This creates a new TJPMFontList}
          constructor Create; reintroduce;
          {: This creates a new TJFontItem and adds it to the TJPMFontList collection}
          function Add: TJFontItem;
          {: This function returns true if the Font AName is in the collection or false if it is not in the collection}
          function IsFontListed (AName: string): Boolean;
          {: This function returns the index of a Font name specified by AName or -1 if the Font was not found}
          function FontIndex (AName: string): Integer;
          {: This property is the TJFontItem in the collection}
          property Items [Index: Integer]: TJFontItem read GetItem write SetItem; default;
     end;

var
     {: This collection lists all of the Fonts for the screeen}
     ScreenFonts: TJPMFontList;
     {: This collections lists all of the Fonts for the printer}
     PrinterFonts: TJPMFontList;

     {: This procedure enumerates all of the screen-Fonts and places them in
     ScreenFonts }
procedure InitScreenFonts;
{: This procedure enumerates all of the default printer Fonts and places them
in ScreenFonts }
procedure InitPrinterFonts;

implementation

function GetDefaultPrinterName: string;
var
     PrinterNameBuf: array [0..MAX_PATH] of char;
begin
     Result := '';
     GetProfileString ('windows', 'device', PChar (''), PrinterNameBuf, MAX_PATH);
     Result := PrinterNameBuf;
     if Length (Result) > 0 then
     begin
          SetLength (Result, Pos (',', Result) - 1);
     end;
end;

{Font enumeration Windows callback function}
{IMPORTANT - Since this is called by a Windows dll, do NOT alter it unless you have
thouroughly read the EnumFontFamExProc entry in the WINAPI help file.  If you have
not or are uncomfortable with it, leave the definition alone}

function FontEnumCallbackProc (var LF: TEnumLogFontExA; var TM: TTextMetric;
     FType: Integer; Data: Pointer): Integer; stdcall;
var
     Collection: TJPMFontList;

     procedure AddToList;
     begin
          {add our Font and the Font type to ComboBox}
          //we could have used elfFullName but for some odd reason, the SYstem Font is returned as #14
          {We use if to prevent the Fonts from being listed several times.  TStrings does not have
          a duplicates property}
          if Collection.FontIndex (LF.elfLogFont.lfFaceName) = -1 then
          begin
               with Collection.Add do
               begin
                    TextMetric := TM;
                    EnumLogFontEx := LF;
                    FontTypeInt := FType;
               end; // with Collection.Add do
          end; // idx = -1
     end; // Procedure AddToList;

begin
     Collection := TJPMFontList (Data);
     AddToList;
     Result := 1;
end;

procedure InitializeLf (var Lf: TLogFont);
{This initializes the LF parameter for Font enumeration}
begin
     LF.lfCharSet := DEFAULT_CHARSET; //indicate that we want all Fonts
     LF.lfFaceName := ''; //we want all TypeFace names
     LF.lfPitchAndFamily := 0; //we have to set this to 0 as instructed by the WinAPI documentation
end;

procedure EnumScreenFonts (ACollection: TJPMFontList);
var
     HDC: THandle;
     LF: TLogFont;
begin
     HDC := 0;
     InitializeLF (LF);
     try
          //allocate a device context for the screen
          HDC := CreateDC (PChar ('DISPLAY'), nil, nil, nil);
          if HDC <> 0 then //do our enumeration
          begin
               EnumFontFamiliesEx (HDC, LF, @FontEnumCallbackProc, DWord (ACollection), 0);
          end;
     finally
          if HDC <> 0 then //free the device context if it is not nil
               DeleteDC (HDC);
     end; //try .. finally
end;

procedure EnumPrinterFonts (ACollection: TJPMFontList);
var
     HDC: THandle;
     LF: TLogFont;
begin
     HDC := 0;
     InitializeLF (LF);
     try
          //allocate a device context for the screen
          HDC := CreateDC (nil, PChar (GetDefaultPrinterName), nil, nil);
          if HDC <> 0 then //do our enumeration
          begin
               EnumFontFamiliesEx (HDC, LF, @FontEnumCallbackProc, DWord (ACollection), 0);
          end;
     finally
          if HDC <> 0 then //free the device context if it is not nil
               DeleteDC (HDC);
     end; //try .. finally
end;

procedure InitScreenFonts;
begin
     ScreenFonts.Clear;
     EnumScreenFonts (ScreenFonts);
end;

procedure InitPrinterFonts;
begin
     PrinterFonts.Clear;
     EnumPrinterFonts (PrinterFonts);
end;
{ TJPMFontList }

function TJPMFontList.Add: TJFontItem;
begin
     Result := TJFontItem (inherited Add);
end;

constructor TJPMFontList.Create;
begin
     inherited Create (TJFontItem);
end;

function TJPMFontList.FontIndex (AName: string): Integer;
var
     idx: Integer;
begin
     Result := -1;
     idx := 0;
     while (idx < Count) and (Result = -1) do
     begin
          if (Length (Items [idx].FontName) = Length (AName)) and (Items [idx].FontName = AName) then
          begin
               Result := Idx;
          end;
          inc (idx);
     end;
end;

function TJPMFontList.GetItem (Index: Integer): TJFontItem;
begin
     Result := TJFontItem (inherited GetItem (Index));
end;

function TJPMFontList.IsFontListed (AName: string): Boolean;
var
     idx: Integer;
begin
     Result := False;
     idx := 0;
     while (idx < Count) and not Result do
     begin
          if (Length (Items [idx].FontName) = Length (AName)) and (Items [idx].FontName = AName) then
               Result := True;
          inc (idx);
     end;
end;

procedure TJPMFontList.SetItem (Index: Integer; const Value: TJFontItem);
begin
     inherited SetItem (Index, Value);
end;

{ TJFontItem }

function TJFontItem.GetFamily: TjpmFontFamily;
begin
     //240 bits 4-7
     case (EnumLogFontEx.elfLogFont.lfPitchAndFamily and (not GetInternalFontPitch)) of
          FF_DECORATIVE: Result := ffDecorative;
          FF_DontCARE: Result := ffUnknown;
          FF_MODERN: Result := ffModern;
          FF_ROMAN: Result := ffRoman;
          FF_SCRIPT: Result := ffScript;
          FF_SWISS: Result := ffSwiss;
     else
          Result := ffUnknown;
     end;
end;

function TJFontItem.GetFontName: string;
begin
     Result := EnumLogFontEx.elfLogFont.lfFaceName;
end;

function TJFontItem.GetFontPitch: TjpmFontPitch;
begin
     case GetInternalFontPitch of
          FIXED_PITCH: Result := fpFixed;
          VARIABLE_PITCH: Result := fpVariable;
          DEFAULT_PITCH: Result := fpDefault;
     else
          Result := fpDefault;
     end;
end;

function TJFontItem.GetFontType: TjpmFontype;
begin
     case FontTypeInt of
          DEVICE_FontTYPE: Result := ftDevice;
          RASTER_FontTYPE: Result := ftRaster;
          TRUETYPE_FontTYPE: Result := ftTrueType;
     else
          Result := ftOther;
     end;
end;

function TJFontItem.GetInternalFontPitch: Byte;
begin
     Result := EnumLogFontEx.elfLogFont.lfPitchAndFamily and 3;
end;

initialization
     ScreenFonts := TJPMFontList.Create;
     EnumScreenFonts (ScreenFonts);
     PrinterFonts := TJPMFontList.Create;
     EnumPrinterFonts (PrinterFonts);
finalization
     ScreenFonts.Free;
     PrinterFonts.Free;
end.
