{: This unit contains the TJPMFontListObject which is a Font selection object
 that is used with selection controls such as a listbox or combobox.
}

{Revision notes:

1-27-2000 - started unit with TjpmDevices
1-28-2000 - changed TjpmDevices to a set
            added TjpmFontypes and TjpmFontype for FontTypes property
1-30-2000 - renamed unit to jfntlstobj to reflect new functionality
2-2-2000 - removed the UseSystemColors property and variable.  It was a duplicate
           of ShowSystemColors and was related to problem where SystemColors were
           not displayed in the IDE
7-28-2000 - made a slight code change to give the preview box outline a more
            consistant "feel"
          - cleaned-up the definition so that it is more readable
7-29-2000 - Added FontPitch property
8-8-2000 - Modified to use the JFontCol unit}
unit JPMFontListObject;

interface

{$I JPM.INC}

uses Classes, Graphics, JPMFontList, JPMGenListObject,
     {$IFDEF BelowD5}StdCtrls, {$ENDIF}
     Windows;

type
     {This type indicates a device used to enumerate Fonts from}
     TjpmDevice = (dScreen, dPrinter);
     {This type is used to indicate the devices to enumerate from from.}
     TjpmDevices = set of TjpmDevice;

     TJPMFontListObject = class (TJPMGenListObject)
     protected
          { Device type for Font selection}
          fDevices: TjpmDevices;
          { Font type for Font selection}
          fFontTypes: TjpmFontypes;
          { Selected Font face }
          fSelectedFont: string;
          fFontSample: string;
          FFontPitches: TjpmFontPitches;
          { SelectedFont property write procedure }
          procedure SetSelectedFont (Value: string);
          procedure SetDevices (Value: TjpmDevices);
          procedure SetFontTypes (Value: TjpmFontypes);
          procedure SetFontPitches (Value: TjpmFontPitches);
          procedure SetFontSample (Value: string);
          { Method to set variable and property values and create objects }
          procedure AutoInitialize; override;
          { Method to free any objects created by AutoInitialize }
          procedure AutoDestroy; override;
          procedure CanvasDrawStyle (const ACanvas: TCanvas;
               AIndex: Integer); override;
     public
          {: This is used in descendants for populating the list control, this one just clears the Items}
          procedure PopulateComboBox; override;
          {: This will draw a box in DrawItem overrides, this abstract version just sets Rect.Left to Rect.Left + 40 if ShowBox}
          procedure DrawBox (Canvas: TCanvas; Index: Integer; var Rect: TRect; State: TOwnerDrawState); override;
          {: This property determines the devices we enumerate Fonts for}
          property Devices: TjpmDevices read FDevices write SetDevices default [dScreen, dPrinter];
          {: Filter for FontTypes that we wish to display}
          property FontTypes: TjpmFontypes read FFontTypes write SetFontTypes default [ftDevice, ftRaster, ftTrueType, ftOther];
          {: The Font face we selected}
          property SelectedFont: string read fSelectedFont write SetSelectedFont;
          {: This is displayed in the various Fonts listed in the control in the Preview box}
          property FontSample: string read fFontSample write SetFontSample;
          {: This is the pitches for the Fonts we wish to display}
          property FontPitches: TjpmFontPitches read FFontPitches write SetFontPitches;
     end;

implementation

{ TJPMFontListObject }

procedure TJPMFontListObject.AutoDestroy;
begin
     inherited
end;

procedure TJPMFontListObject.AutoInitialize;
begin
     inherited AutoInitialize;
     fDevices := [dScreen, dPrinter];
     fFontTypes := [ftDevice, ftRaster, ftTrueType, ftOther];
     FFontPitches := [fpFixed, fpVariable, fpDefault];
     fSelectedFont := ''; //just to prevent corruption
     FFontSample := 'AaBb';
end;

procedure TJPMFontListObject.CanvasDrawStyle (const ACanvas: TCanvas;
     AIndex: Integer);
begin
     inherited;
     if FShowStyledText then
     begin
          ACanvas.Font.Name := Items [AIndex];
     end;
end;

procedure TJPMFontListObject.DrawBox (Canvas: TCanvas; Index: Integer;
     var Rect: TRect; State: TOwnerDrawState);
var
     stTempFontFace: string;
begin
     if ShowBox = False then
          Exit;
     {Font Preview rectangle}
     DrawBoxOutLine (Canvas, Index, Rect, State);
     stTempFontFace := Canvas.Font.Name; {temporarily store Font name}
     Canvas.Font.Name := Items [Index]; //display text in the Font listed in combobox
     if ShowBoxBorder then
          Canvas.TextRect (Classes.Rect (Rect.Left + 2, Rect.Top + 2, FShowBoxWidth - 1, Rect.Bottom - 2), Rect.Left + 2, Rect.Top + 2, FFontSample)
     else
          Canvas.TextRect (Classes.Rect (Rect.Left + 1, Rect.Top + 1, FShowBoxWidth, Rect.Bottom - 1), Rect.Left + 1, Rect.Top + 1, FFontSample);
     Canvas.Font.Name := stTempFontFace; {restore Font name}
     inherited DrawBox (Canvas, Index, Rect, State);
end;

procedure TJPMFontListObject.PopulateComboBox;
var
     idx: Integer;
begin
     inherited PopulateComboBox;
     {Screen Fonts}
     if dScreen in fDevices then
     begin
          idx := 0;
          while idx < ScreenFonts.Count do
          begin
               if (ScreenFonts [idx].FontPitch in FontPitches) and
                    (ScreenFonts [idx].FontType in FontTypes) then
               begin
                    Items.Add (ScreenFonts [idx].FontName);
               end; //if
               Inc (idx);
          end;
     end; //if dScreen in fDevices then
     {Printer Fonts}
     if dPrinter in fDevices then
     begin
          idx := 0;
          while idx < PrinterFonts.Count do
          begin
               if (PrinterFonts [idx].FontPitch in FontPitches) and
                    (PrinterFonts [idx].FontType in FontTypes) then
               begin
                    if Items.IndexOf (PrinterFonts [idx].FontName) = -1 then
                         Items.Add (PrinterFonts [idx].FontName);
               end; //if
               Inc (idx);
          end;
     end; //if dPrinter in fDevices then
end;

procedure TJPMFontListObject.SetDevices (Value: TjpmDevices);
begin
     FDevices := Value;
end;

procedure TJPMFontListObject.SetFontPitches (Value: TjpmFontPitches);
begin
     fFontPitches := Value;
end;

procedure TJPMFontListObject.SetFontSample (Value: string);
begin
     fFontSample := Value;
end;

procedure TJPMFontListObject.SetFontTypes (Value: TjpmFontypes);
begin
     FFontTypes := Value;
end;

procedure TJPMFontListObject.SetSelectedFont (Value: string);
begin
     fSelectedFont := Value;
end;

end.
