{: Supplies Registry/IniFile Support for JPM Open.

 Copyright  2000-2001 by J. Peter Mugaas

}
unit JPMRegIni;

interface

const
     {: Default Registry Path to use }
     DefRegPath = '\Software\JPM';
     {: Default Registry Section to use }
     DefRegSection = 'Colors';
     {: Default IniFile to use }
     DefIniFile: string = 'jpm.ini';
     {: Default IniFile Section to use }
     DefIniSection: string = 'Colors';

     {: Saves <See Var=JPMCustomColors> to Registry using the specified Path
      & Section. }
procedure SaveCustomColorsToRegistry (const RegPath, RegSection: string);

{: Loads <See Var=JPMCustomColors> from the Registry using the specified Path
 & Section. }
procedure LoadCustomColorsFromRegistry (const RegPath, RegSection: string);

{: Saves <See Var=JPMCustomColors> to an IniFile using the specified Path
 & Section. }
procedure SaveCustomColorsToIniFile (const IniPath, IniSection: string);

{: Loads <See Var=JPMCustomColors> from an IniFile using the specified Path
 & Section. }
procedure LoadCustomColorsFromIniFile (const IniPath, IniSection: string);

implementation

uses
     SysUtils, IniFiles, JPMColorListObject, Registry;

procedure SaveCustomColorsToRegistry (const RegPath, RegSection: string);
var
     Reg: TRegistryIniFile;
     I: Integer;
begin
     if (RegPath <> '') and (RegSection <> '') then
     begin
          Reg := TRegistryIniFile.Create (RegPath);
          try
               for I := 0 to JPMCustomColors.Count - 1 do
               begin
                    Reg.WriteString (RegSection, 'Color' + IntToStr (I),
                         JPMCustomColors [I]);
               end;
          finally
               Reg.Free
          end;
     end;
end;

procedure LoadCustomColorsFromRegistry (const RegPath, RegSection: string);
var
     Reg: TRegistryIniFile;
     I: Integer;
     S: string;
begin
     if (RegPath <> '') and (RegSection <> '') then
     begin
          Reg := TRegistryIniFile.Create (RegPath);
          try
               JPMCustomColors.Clear;
               for I := 0 to 16 do
               begin
                    S := Reg.ReadString (RegSection, 'Color' + IntToStr (I), '');
                    if S <> '' then
                         JPMCustomColors.Add (S);
               end;
          finally
               Reg.Free
          end;
     end;
end;

procedure SaveCustomColorsToIniFile (const IniPath, IniSection: string);
var
     Ini: TIniFile;
     I: Integer;
begin
     if (IniPath <> '') and (IniSection <> '') then
     begin
          Ini := TIniFile.Create (IniPath);
          try
               for I := 0 to JPMCustomColors.Count - 1 do
               begin
                    Ini.WriteString (IniSection, 'Color' + IntToStr (I),
                         JPMCustomColors [I]);
               end;
          finally
               Ini.Free
          end;
     end;
end;

procedure LoadCustomColorsFromIniFile (const IniPath, IniSection: string);
var
     Ini: TIniFile;
     I: Integer;
     S: string;
begin
     if (IniPath <> '') and (IniSection <> '') then
     begin
          Ini := TIniFile.Create (IniPath);
          try
               JPMCustomColors.Clear;
               for I := 0 to 16 do
               begin
                    S := Ini.ReadString (IniSection, 'Color' + IntToStr (I), '');
                    if S <> '' then
                         JPMCustomColors.Add (S);
               end;
          finally
               Ini.Free
          end;
     end;
end;

end.
