unit JPMutils;
{: Supplies some useful routines for JPM Open.

 Copyright  2000 by J. Peter Mugaas

}

interface

{$I JPM.INC}

uses Windows;

//: This returns the work area on the user's screen (stuff not covered by the taskbar
function GetWorkArea: TRect;
//: This returns the border tickness of the Windows border
function GetBorderTickness: Integer;
//: This returns the witdth of the vertical scroll bar
function GetScrollVWidth: Integer;
//: This returns the height of the horizontal scroll bar
function GetScrollHHeight: Integer;

//: This returns the width of Caption button such as the Minimize Button
function GetCaptionBtnWidth: Integer;
//: This returns the hieght of Caption button such as the Minimize Button
function GetCaptionBtnHieght: Integer;

//: This returns True if Icon Titles are wrapped or false if they are not wrapped
function IcontitleWrapped: Boolean;
//:  This is the icon horizontal spacing in Pixels
function IconHSpacing: Integer;
//:  This is the icon vertical spacing in Pixels
function IconVSpacing: Integer;

//: Windows 95, 98, and 2000 only - This returns true if High Contrast is on or False if it is off
function IsHighContrastOn: Boolean;
//: Windows 95, 98, and 2000 only - This returns true if High Contrast is available
function IsHighContrastAvailable: Boolean;
//: Windows 95, 98, and 2000 only - This returns True if the Hokey prompts for a confirmation dialog-box
function IsHighContrastHotKeyConfirmDlg: Boolean;
//: Windows 95, 98, and 2000 only - This returns True if the user can switch High-Contrast on or off with a HotKey
function IsHighContrastHotKeyActive: Boolean;
//: Windows 95, 98, and 2000 only - This returns True if the hot key associated with the High Contrast Mode can be enabled. An application can retrieve this value, but cannot set it.
function IsHighContrastHotKeyAvailable: Boolean;
//: Windows 95, 98, and 2000 only - This returns True if a sound is played when the hotkey is toggled
function IsHighContrastHotKeySound: Boolean;

//: Windows 95, 98, and 2000 only - This returns True if there is a screen-reader
function IsScreenReader: Boolean;
//: Windows 95, 98, and 2000 only - This returns True if the user relies on the keyboard
function UserReliesOnKeyboard: Boolean;
//: Windows 95, 98, and 200 only - This returns True if there is a screen-reader is in use
function ScreenReaderInUse: Boolean;
//: This returns true if the show sounds accessibility feature is on.
function ShowSound: Boolean;

//: This type indicates the Operating System on the user's computer in the TOSVerInfo object
type
     TJPMWin32Type = (Win32s, WindowsNT40, Windows95, Windows95OSR2, Windows98, Windows98SE, WindowsMe, Windows2000Pro, Windows2000Server, Windows200DomainController, WindowsXP);
     //: This type is used in the TJPMInstalledSuites set and indicates a Windows 2000
     TJPMInstalledSuite = (BackOffice, DataCenter, AdvancedServer, SmallBusinessServer, SmallBusinessServerRestrictive, TerminalServices);
     //: This set indicates the installed suites in Windows 2000 in the TOSVerInfo object
     TJPMInstalledSuites = set of TJPMInstalledSuite;

     //: This object encapsolates information about the Operating System
type
     TOSVerInfo = class (TObject)
     private
     protected
          FOperatingSystem: TJPMWin32Type;
          FInstalledSuites: TJPMInstalledSuites;
          FServicePack: string;
          FMajorVersion,
               fMinorVersion,
               FBuild,
               FPlatFormId: Uint;
          function GetIsWindowsNT: Boolean;
     public
          constructor Create;
          //: This is the operating system as we could determine it
          property OperatingSystem: TJPMWin32Type read FOperatingSystem;
          //: This is the installed Suites on Windows 2000 or greater
          property InstalledSuites: TJPMInstalledSuites read FInstalledSuites;
          //: This is the service pack for Windows NT 4.0 or Windows 2000
          property ServicePack: string read FServicePack;
          //: This is the major version from the GetVersionEx Win32 API call
          property MajorVersion: UInt read FMajorVersion;
          //: This is the minor version of the GetVersionEx Win32 API call
          property MinorVersion: UInt read fMinorVersion;
          //: This is the build number from the GetVersionEx Win32 API call
          property Build: UInt read FBuild;
          //: This is the platform ID from the GetVersionEx Win32 API call
          property PlatFormId: UInt read FPlatFormId;
          //: This returns True if the platform is WindowsNT 4.0 or Win32s - This is needed because some accessibility functions are not supported on those platforms
          property IsWindowsNT40: Boolean read GetIsWindowsNT;
     end;

     //: This is an internal function which we make public to facilitate JPMGraphics
function GetTNonClientMetrics: TNonClientMetrics;
//: This is an internal function which we make public to facilitate JPMGraphics
function GettagICONMETRICS: tagICONMETRICS;

//: This object provides information about the user's operating system.
var
     Ver: TOSVerInfo;

implementation
uses JPMWinApiEx, SysUtils;

function GetWorkArea: TRect;
begin
     SystemParametersInfo (SPI_GETWORKAREA,
          sizeof (TRect), @Result, 0);
end;

function GetBorderTickness: Integer;
begin
     Result := GetTNonClientMetrics.iBorderWidth;
end;

function GetScrollVWidth: Integer;
begin
     Result := GetTNonClientMetrics.iScrollWidth;
end;

function GetScrollHHeight: Integer;
begin
     Result := GetTNonClientMetrics.iScrollHeight;
end;

function GetCaptionBtnWidth: Integer;
begin
     Result := GetTNonClientMetrics.iCaptionWidth;
end;

function GetCaptionBtnHieght: Integer;
begin
     Result := GetTNonClientMetrics.iCaptionHeight;
end;

{This get's some WIN32 API types}

{Leave this as long bool because Win 32 API functions are very picky about this}

function GetBoolean (AParam: UInt): LongBool;
begin
     SystemParametersInfo (AParam,
          sizeof (LongBool), @Result, 0);
end;

function GetBooleanSysMetrics (AParam: UInt): Boolean;
begin
     Result := Boolean (GetSystemMetrics (AParam));
end;

function GetTNonClientMetrics: TNonClientMetrics;
begin
     Result.cbSize := sizeof (TNonClientMetrics);
     SystemParametersInfo (SPI_GETNONCLIENTMETRICS,
          sizeof (TNonClientMetrics), @Result, 0);
end;

function GettagHIGHContRAST: tagHIGHContRAST;
begin
     Result.cbSize := SizeOf (tagHIGHContRAST);
     SystemParametersInfo (SPI_GETHIGHContRAST,
          sizeof (tagHIGHContRAST), @Result, 0);
end;

function GettagICONMETRICS: tagICONMETRICS;
begin
     Result.cbSize := sizeof (tagICONMETRICS);
     SystemParametersInfo (SPI_GETICONMETRICS,
          sizeof (tagICONMETRICS), @Result, 0);
end;

{I had to comment this out because it would trigger warnings - I will use it in a future version
Function GettagACCESSTIMEOUT : tagACCESSTIMEOUT;
begin
  Result.cbSize := sizeof(tagACCESSTIMEOUT);
  SystemParametersInfo(SPI_GETNONCLIENTMETRICS,
    sizeof(tagACCESSTIMEOUT), @Result, 0);
end;}

{Icon info }

function IcontitleWrapped: Boolean;
begin
     Result := GettagICONMETRICS.iTitleWrap <> 0;
end;

function IconHSpacing: Integer;
begin
     Result := GettagICONMETRICS.iHorzSpacing;
end;

function IconVSpacing: Integer;
begin
     Result := GettagICONMETRICS.iVertSpacing;
end;

{accessibility}

function IsHighContrastOn: Boolean;
begin
     if Ver.IsWindowsNT40 then
          Result := (GettagHIGHContRAST.dwFlags and HCF_HIGHContRASTON = HCF_HIGHContRASTON)
     else
          Result := False;
end;

function IsHighContrastAvailable: Boolean;
begin
     if Ver.IsWindowsNT40 then
          Result := (GettagHIGHContRAST.dwFlags and HCF_AVAILABLE = HCF_AVAILABLE)
     else
          Result := False;
end;

function IsHighContrastHotKeyConfirmDlg: Boolean;
begin
     if Ver.IsWindowsNT40 then
          Result := (GettagHIGHContRAST.dwFlags and HCF_CONFIRMHOTKEY = HCF_CONFIRMHOTKEY)
     else
          Result := False;
end;

function IsHighContrastHotKeyActive: Boolean;
begin
     if Ver.IsWindowsNT40 then
          Result := (GettagHIGHContRAST.dwFlags and HCF_HOTKEYACTIVE = HCF_HOTKEYACTIVE)
     else
          Result := False;
end;

function IsHighContrastHotKeyAvailable: Boolean;
begin
     if Ver.IsWindowsNT40 then
          Result := (GettagHIGHContRAST.dwFlags and HCF_HOTKEYAVAILABLE = HCF_HOTKEYAVAILABLE)
     else
          Result := False;
end;

function IsHighContrastHotKeySound: Boolean;
begin
     if Ver.IsWindowsNT40 then
          Result := (GettagHIGHContRAST.dwFlags and HCF_HOTKEYSOUND = HCF_HOTKEYSOUND)
     else
          Result := False;
end;

function IsScreenReader: Boolean;
begin
     if Ver.IsWindowsNT40 then
          Result := GetBoolean (SPI_GETSCREENREADER)
     else
          Result := False;
end;

function UserReliesOnKeyboard: Boolean;
begin
     Result := GetBoolean (SPI_GETKEYBOARDPREF);
end;

function ScreenReaderInUse: Boolean;
begin
     if Ver.IsWindowsNT40 then
          Result := GetBoolean (SPI_GETSCREENREADER)
     else
          Result := False;
end;

function ShowSound: Boolean;
begin
     Result := GetBooleanSysMetrics (SM_SHOWSOUNDS);
end;

{ TOSVerInfo }

constructor TOSVerInfo.Create;
var
     VerInfo: TOSVersionInfoEx2;
begin
     inherited Create;
     VerInfo.dwOSVersionInfoSize := SizeOf (TOSVersionInfo);
     GetVersionEx2 (VerInfo);
     if VerInfo.dwMajorVersion >= 5 then
          {Windows 2000, use extended call}
     begin
          VerInfo.dwOSVersionInfoSize := SizeOf (TOSVersionInfoEx2);
          GetVersionEx2 (VerInfo);
          case VerInfo.bwProductType of
               VER_NT_WORKSTATION: FOperatingSystem := Windows2000Pro;
               VER_NT_DOMAIN_ContROLLER: FOperatingSystem := Windows200DomainController;
               VER_NT_SERVER: FOperatingSystem := Windows2000Server;
          end;
          if VerInfo.wSuiteMask and VER_SUITE_BACKOFFICE = VER_SUITE_BACKOFFICE then
               FInstalledSuites := FInstalledSuites + [BackOffice];
          if VerInfo.wSuiteMask and VER_SUITE_DATACENTER = VER_SUITE_DATACENTER then
               FInstalledSuites := FInstalledSuites + [DataCenter];
          if VerInfo.wSuiteMask and VER_SUITE_ENTERPRISE = VER_SUITE_ENTERPRISE then
               FInstalledSuites := FInstalledSuites + [AdvancedServer];
          if VerInfo.wSuiteMask and VER_SUITE_SMALLBUSINESS = VER_SUITE_SMALLBUSINESS then
               FInstalledSuites := FInstalledSuites + [SmallBusinessServer];
          if VerInfo.wSuiteMask and VER_SUITE_SMALLBUSINESS_RESTRICTED = VER_SUITE_SMALLBUSINESS_RESTRICTED then
               FInstalledSuites := FInstalledSuites + [SmallBusinessServerRestrictive];
          if VerInfo.wSuiteMask and VER_SUITE_TERMINAL = VER_SUITE_TERMINAL then
               FInstalledSuites := FInstalledSuites + [TerminalServices];
          if (VerInfo.wServicePackMajor <> 0) or (VerInfo.wServicePackMinor <> 0) then
               FServicePack := Format ('Service Pack %d.%d', [VerInfo.wServicePackMajor, VerInfo.wServicePackMinor]);
          if VerInfo.dwMinorVersion > 0 then
          begin
               FOperatingSystem := WindowsXP;
          end;
     end
     else
     begin
          {is this WIndows 95, 98, Me, or NT 40}
          if VerInfo.dwMajorVersion > 3 then
          begin
               if VerInfo.dwPlatformId = VER_PLATFORM_WIN32_NT then
               begin
                    FOperatingSystem := WindowsNT40;
                    FServicePack := VerInfo.szCSDVersion;
               end
               else
               begin
                    {mask off junk}
                    VerInfo.dwBuildNumber := VerInfo.dwBuildNumber and $FFFF;

                    if VerInfo.dwMinorVersion >= 90 then
                         FOperatingSystem := WindowsMe
                    else if VerInfo.dwMinorVersion >= 10 then
                    begin
                         {Windows 98}
                         if VerInfo.dwBuildNumber >= 2222 then
                              FOperatingSystem := Windows98SE
                         else
                              FOperatingSystem := Windows98;
                    end
                    else
                    begin
                         {Windows 95}
                         if VerInfo.dwBuildNumber >= 1000 then
                              FOperatingSystem := Windows95OSR2
                         else
                              FOperatingSystem := Windows95;
                    end;
               end;
          end
          else
          begin
               FOperatingSystem := Win32s;
          end;
     end;
     FMajorVersion := VerInfo.dwMajorVersion;
     fMinorVersion := VerInfo.dwMinorVersion;
     FBuild := VerInfo.dwBuildNumber;
     FPlatFormId := VerInfo.dwPlatformId;
end;

function TOSVerInfo.GetIsWindowsNT: Boolean;
begin
     Result := (FOperatingSystem = WindowsNT40) or (FOperatingSystem = Win32s);
end;

initialization
     ver := TOSVerInfo.Create;
finalization
     ver.Free;
     ver := nil;
end.
