unit JRCheckBox;

{$R-}
{$R JRCheckBox.res}

interface

uses
  Windows, SysUtils, Classes, Controls, StdCtrls, ExtCtrls, Messages, Graphics,
  ImgList, Dialogs;

type
  TJRCheckBox = class(TCheckBox)
  private
    { Private declarations }
    FOnEnterTextFont: TFont;
    FOnExitTextFont: TFont;
    FAllowStandardization: boolean;
    FFlat: Boolean;
    FMultiLine: Boolean;
    FPushLike: Boolean;
    procedure SetFlat(const Value: Boolean);
    procedure SetMultiLine(const Value: Boolean);
    procedure SetPushLike(const Value: Boolean);
    procedure SetOnEnterTextFont(Value: TFont);
    procedure SetAllowStandardization(Value: boolean);
  protected
    { Protected declarations }
    procedure CMEnter(var Message: TCMGotFocus); message CM_ENTER;
    procedure CMExit(var Message: TCMLostFocus); message CM_EXIT;
    procedure CreateParams(var Params: TCreateParams); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent);override;
  published
    { Published declarations }
    property OnEnterTextFont: TFont read FOnEnterTextFont write SetOnEnterTextFont;
    property AllowStandardization: boolean read FAllowStandardization
      write SetAllowStandardization;
    property Flat: Boolean read FFlat write SetFlat default true;
    property MultiLine: Boolean read FMultiLine write SetMultiLine default false;
    property PushLike: Boolean read FPushLike write SetPushLike default false;
  end;

implementation

uses
  JRGrUtilities;

constructor TJRCheckBox.Create(AOwner: TComponent);
begin
  FFlat := true;
  FMultiLine := false;
  FPushLike := false;
  inherited Create(AOwner);
  FOnEnterTextFont := TFont.Create;
  FOnExitTextFont := TFont.Create;
  with FOnEnterTextFont do begin
    Color := clBlue;
    Style := [fsBold];
  end;
end;

procedure TJRCheckBox.SetOnEnterTextFont(Value: TFont);
begin
  jruCopiarFontes(FOnEnterTextFont, Value);
end;

procedure TJRCheckBox.SetAllowStandardization(Value: boolean);
begin
  if Value <> FAllowStandardization then
    FAllowStandardization := Value;
end;

procedure TJRCheckBox.CMEnter(var Message: TCMGotFocus);
begin
  jruCopiarFontes(FOnExitTextFont, Font);
  jruCopiarFontes(Font, FOnEnterTextFont);
end;

procedure TJRCheckBox.CMExit(var Message: TCMLostFocus);
begin
  jruCopiarFontes(Font, FOnExitTextFont);
end;

procedure TJRCheckBox.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  if FFlat then
    Params.Style := Params.Style or BS_FLAT
  else
    Params.Style := Params.Style and not BS_FLAT;
  if FMultiLine then
    Params.Style := Params.Style or BS_MULTILINE
  else
    Params.Style := Params.Style and not BS_MULTILINE;
  if FPushLike then
    Params.Style := Params.Style or BS_PUSHLIKE
  else
    Params.Style := Params.Style and not BS_PUSHLIKE;
end;


procedure TJRCheckBox.SetFlat(const Value: Boolean);
begin
  if Value <> FFlat then
  begin
    FFlat := Value;
    RecreateWnd;
  end;
end;


procedure TJRCheckBox.SetMultiLine(const Value: Boolean);
begin
  if Value <> FMultiLine then
  begin
    FMultiLine := Value;
    RecreateWnd;
  end;
end;


procedure TJRCheckBox.SetPushLike(const Value: Boolean);
begin
  if Value <> FPushLike then
  begin
    FPushLike := Value;
    RecreateWnd;
  end;
end;

end.
