unit JRDBCheckBoxMySQL;

{$R-}
{$R JRDBCheckBoxMySQL.res}

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, StdCtrls, JRCheckBox, DBCtrls, DB;

type
  TJRDBCheckBoxMySQL = class(TJRCheckBox)
  private
    FDataLink: TFieldDataLink;
    FPaintControl: TPaintControl;
    procedure DataChange(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    function GetFieldState: TCheckBoxState;
    function GetReadOnly: Boolean;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure SetReadOnly(Value: Boolean);
    procedure UpdateData(Sender: TObject);
    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure CMGetDataLink(var Message: TMessage); message CM_GETDATALINK;
  protected
    procedure Toggle; override;
    procedure KeyPress(var Key: Char); override;
    procedure Notification(AComponent: TComponent;
      Operation: TOperation); override;
    procedure WndProc(var Message: TMessage); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function ExecuteAction(Action: TBasicAction): Boolean; override;
    function UpdateAction(Action: TBasicAction): Boolean; override;
    function UseRightToLeftAlignment: Boolean; override;
    property Checked;
    property Field: TField read GetField;
    property State;
  published
    property Action;
    property Alignment;
    property AllowGrayed;
    property Anchors;
    property BiDiMode;
    property Caption;
    property Color;
    property Constraints;
    property Ctl3D;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    property DragCursor;
    property DragKind;
    property DragMode;
    property Enabled;
    property Font;
    property ParentBiDiMode;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ReadOnly: Boolean read GetReadOnly write SetReadOnly default False;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnClick;
    property OnContextPopup;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnStartDock;
    property OnStartDrag;
  end;

implementation

constructor TJRDBCheckBoxMySQL.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  State := cbUnchecked;
  FDataLink := TFieldDataLink.Create;
  FDataLink.Control := Self;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnUpdateData := UpdateData;
  FPaintControl := TPaintControl.Create(Self, 'BUTTON');
  FPaintControl.Ctl3DButton := True;
end;

destructor TJRDBCheckBoxMySQL.Destroy;
begin
  FPaintControl.Free;
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
end;

procedure TJRDBCheckBoxMySQL.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
end;

function TJRDBCheckBoxMySQL.UseRightToLeftAlignment: Boolean;
begin
  Result := DBUseRightToLeftAlignment(Self, Field);
end;

function TJRDBCheckBoxMySQL.GetFieldState: TCheckBoxState;
begin
  if (FDataLink <> nil)and(FDatalink.Field <> nil) then
  begin
    if FDataLink.Field.AsString = '' then
    begin
      if AllowGrayed then
        Result := cbGrayed
      else
        Result := cbUnchecked
    end
    else if FDataLink.Field.AsInteger = 1 then
      Result := cbChecked
    else
      Result := cbUnchecked
  end
  else
      if AllowGrayed then
        Result := cbGrayed
      else
        Result := cbUnchecked
end;

procedure TJRDBCheckBoxMySQL.DataChange(Sender: TObject);
begin
  State := GetFieldState;
end;

procedure TJRDBCheckBoxMySQL.UpdateData(Sender: TObject);
begin
  if State = cbGrayed then
    FDataLink.Field.AsString := ''
  else if Checked then
    FDataLink.Field.AsInteger := 1
  else
    FDataLink.Field.AsInteger := 0;
end;

procedure TJRDBCheckBoxMySQL.Toggle;
begin
  if FDataLink.Edit then
  begin
    inherited Toggle;
    FDataLink.Modified;
  end;
end;

function TJRDBCheckBoxMySQL.GetDataSource: TDataSource;
begin
  Result := FDataLink.DataSource;
end;

procedure TJRDBCheckBoxMySQL.SetDataSource(Value: TDataSource);
begin
  if not (FDataLink.DataSourceFixed and (csLoading in ComponentState)) then
    FDataLink.DataSource := Value;
  if Value <> nil then Value.FreeNotification(Self);
end;

function TJRDBCheckBoxMySQL.GetDataField: string;
begin
  Result := FDataLink.FieldName;
end;

procedure TJRDBCheckBoxMySQL.SetDataField(const Value: string);
begin
  FDataLink.FieldName := Value;
end;

function TJRDBCheckBoxMySQL.GetReadOnly: Boolean;
begin
  Result := FDataLink.ReadOnly;
end;

procedure TJRDBCheckBoxMySQL.SetReadOnly(Value: Boolean);
begin
  FDataLink.ReadOnly := Value;
end;

function TJRDBCheckBoxMySQL.GetField: TField;
begin
  Result := FDataLink.Field;
end;

procedure TJRDBCheckBoxMySQL.KeyPress(var Key: Char);
begin
  inherited KeyPress(Key);
  case Key of
    #8, ' ':
      FDataLink.Edit;
    #27:
      FDataLink.Reset;
  end;
end;

procedure TJRDBCheckBoxMySQL.WndProc(var Message: TMessage);
begin
  with Message do
    if (Msg = WM_CREATE) or (Msg = WM_WINDOWPOSCHANGED) or
      (Msg = CM_TEXTCHANGED) or (Msg = CM_FONTCHANGED) then
      FPaintControl.DestroyHandle;
  inherited;
end;

procedure TJRDBCheckBoxMySQL.WMPaint(var Message: TWMPaint);
begin
  if not (csPaintCopy in ControlState) then inherited else
  begin
    SendMessage(FPaintControl.Handle, BM_SETCHECK, Ord(GetFieldState), 0);
    SendMessage(FPaintControl.Handle, WM_PAINT, Message.DC, 0);
  end;
end;

procedure TJRDBCheckBoxMySQL.CMExit(var Message: TCMExit);
begin
  try
    FDataLink.UpdateRecord;
  except
    SetFocus;
    raise;
  end;
  inherited;
end;

procedure TJRDBCheckBoxMySQL.CMGetDataLink(var Message: TMessage);
begin
  Message.Result := Integer(FDataLink);
end;

function TJRDBCheckBoxMySQL.ExecuteAction(Action: TBasicAction): Boolean;
begin
  Result := inherited ExecuteAction(Action) or (FDataLink <> nil) and
    FDataLink.ExecuteAction(Action);
end;

function TJRDBCheckBoxMySQL.UpdateAction(Action: TBasicAction): Boolean;
begin
  Result := inherited UpdateAction(Action) or (FDataLink <> nil) and
    FDataLink.UpdateAction(Action);
end;

end.
