{: This unit contains the TJPMFontEComboBox  which is an ESBPCS dependant Font
selection combo-box}
{Release notes
1-27-2000 Started on componnent
1-28-2000 created a FontTypes selection property
          added a SelectedFont property so you can get the selection the user made
1-30-2000 restructured the code so that it now depends upon TJPMFontListObject for
          Font enumeration and PreviewBox display
          dropped tbe Preview pain visible property in favor of the
          PreviewBoxWidth property
          Added FontSample property so you can customize the string displayed in
          the preview box
1-31-2000 published ItemHeight and Font properties
          Moved component to "JPM Open" tab.
2-2-2000 fixed problem with dynamically-created controls
         fixed problem with a predefined selected Font not being reflected in
         selection if present
7-29-2000 I added a FontPitch property.
          I fixed a problem where Font enumeration was triggered several times.
          I also fixed a problem where an enumeration was triggered in IDE when
          setting properties which is not a good idea for a combobox
8-8-2000 - Modified to use the JFontCol unit}

unit JPMEFontComboBox;

interface

{$I JPM.INC}

uses Classes, Controls, Dialogs,
     Esbpcscombobox, Graphics, JPMFontListObject, JPMFontList,
     Stdctrls, Windows;

type
     TJPMEFontComboBox = class (TESBCustomComboBox)
     private
          { Private fields of TJPMEFontComboBox }
          fFontSelObj: TJPMFontListObject;
          procedure AutoInitialize;
          procedure AutoDestroy;
          procedure PopulateComboBox;
          procedure SetDevices (Value: TjpmDevices);
          function GetDevices: TjpmDevices;
          procedure SetFontTypes (Value: TjpmFontypes);
          function GetFontTypes: TjpmFontypes;
          procedure SetSelectedFont (Value: string);
          function GetSelectedFont: string;
          procedure SetFontSample (Value: string);
          function GetFontSample: string;
          procedure SetFontPitches (Value: TjpmFontPitches);
          function GetFontPitches: TjpmFontPitches;
          {PreviewBoxWidth write method for PreviewBoxWidth }
          procedure SetPreviewBoxWidth (Value: Integer);
          {PreviewBoxWidth read method for PreviewBoxWidth }
          function GetPreviewBoxWidth: Integer;
          function GetJPMVersion: string;
          procedure SetJPMVersion (const Value: string);
          function GetShowBox: Boolean;
          function GetShowBoxBorder: Boolean;
          function GetShowStyledText: Boolean;
          procedure SetShowBox (const Value: Boolean);
          procedure SetShowBoxBorder (const Value: Boolean);
          procedure SetShowStyledText (const Value: Boolean);
     protected
          { Protected fields of TJPMEFontComboBox }
          { Protected methods of TJPMEFontComboBox }
          procedure SetParent (AParent: TWinControl); override;
          procedure Change; override;
          procedure Click; override;
          procedure KeyPress (var Key: Char); override;
          procedure Loaded; override;
     public
          { Public fields and properties of TJPMEFontComboBox }
          { Public methods of TJPMEFontComboBox }
          constructor Create (AOwner: TComponent); override;
          destructor Destroy; override;
          {: Draw override to display Fonts }
          procedure DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState); override;
     published
          { Published properties of TJPMEFontComboBox }
          property ItemHeight;
          property Font;
          property TabStop;
          property TabOrder;
          property Visible;
          property OnChange;
          property OnClick;
          property OnDblClick;
          property OnDragDrop;
          property OnEnter;
          property OnExit;
          property OnKeyDown;
          property OnKeyPress;
          property OnKeyUp;
          property OnMouseDown;
          property OnMouseMove;
          property OnMouseUp;
          {: This property determines the devices we enumerate Fonts for}
          property Width default 126;
          {: This property determines the devices we enumerate Fonts for}
          property Devices: TjpmDevices read GetDevices write SetDevices default [dScreen, dPrinter];
          {: Filter for FontTypes that we wish to display}
          property FontTypes: TjpmFontypes read GetFontTypes write SetFontTypes default [ftDevice, ftRaster, ftTrueType, ftOther];
          {: The Font face we selected}
          property SelectedFont: string read GetSelectedFont write SetSelectedFont
               stored ValueStored;
          {: This is displayed in the various Fonts listed in the control in the Preview box}
          property FontSample: string read GetFontSample write SetFontSample;
          {: Width of the preview box}
          property PreviewBoxWidth: Integer read GetPreviewBoxWidth write SetPreviewBoxWidth default 40;
          {: This is the pitches for the Fonts we wish to display}
          property FontPitches: TjpmFontPitches read GetFontPitches write SetFontPitches;
          {: Show a border around the preview box}
          property ShowBoxBorder: Boolean read GetShowBoxBorder write SetShowBoxBorder;
          {: Show a preview box in ListControl}
          property ShowBox: Boolean read GetShowBox write SetShowBox;
          {: ShowStyledText variable indicates that we should draw text outside the preview with a particular
            Font or color}
          property ShowStyledText: Boolean read GetShowStyledText write SetShowStyledText;
          {: Displays the Current Version of the Component. }
          property Version: string read GetJPMVersion write SetJPMVersion stored false;
     end;

implementation
uses JPMConst;

{ Method to set variable and property values and create objects }

procedure TJPMEFontComboBox.AutoInitialize;
begin
     fFontSelObj := TJPMFontListObject.Create (Items);
     Sorted := True; //we sort the list of Fonts alphabetically
     Width := 126;
     fFontSelObj.SelectedFont := '';
     fFontSelObj.FontSample := 'AaBb';
     fFontSelObj.Devices := [dScreen, dPrinter]; //default to both Screen and Printer
     fFontSelObj.FontTypes := [ftDevice, ftRaster, ftTrueType, ftOther]; //default to all Fonts
end; { of AutoInitialize }

{ Method to free any objects created by AutoInitialize }

procedure TJPMEFontComboBox.AutoDestroy;
begin
     fFontSelObj.Free;
end; { of AutoDestroy }

function TJPMEFontComboBox.GetJPMVersion: string;
begin
     Result := JPMVersion;
end;

procedure TJPMEFontComboBox.SetJPMVersion (const Value: string);
begin
     // Do nothing
end;

{ Override OnChange handler from TESBCustomComboBox }

procedure TJPMEFontComboBox.Change;
begin
     fFontSelObj.SelectedFont := Items [ItemIndex]; //set fSelectedFont when the selection changes
     inherited Change;
end; //procedure TJPMEFontComboBox.Change;

{ Override OnClick handler from TESBCustomComboBox }

procedure TJPMEFontComboBox.Click;
begin
     { Code to execute before activating click
       behavior of component's parent class }

     { Activate click behavior of parent }
     inherited Click;

     { Code to execute after click behavior
       of parent }

end; //procedure TJPMEFontComboBox.Click;

{ Override OnKeyPress handler from TESBCustomComboBox }

procedure TJPMEFontComboBox.KeyPress (var Key: Char);
const
     TabKey = Char (VK_TAB);
     EnterKey = Char (VK_RETURN);
begin
     { Key contains the character produced by the keypress.
       It can be tested or assigned a new value before the
       call to the inherited KeyPress method.  Setting Key
       to #0 before call to the inherited KeyPress method
       terminates any further processing of the character. }

     { Activate KeyPress behavior of parent }
     inherited KeyPress (Key);

     { Code to execute after KeyPress behavior of parent }

end; //procedure TJPMEFontComboBox.KeyPress(var Key : Char);

constructor TJPMEFontComboBox.Create (AOwner: TComponent);
begin
     { Call the Create method of the parent class }
     inherited Create (AOwner);

     { AutoInitialize sets the initial values of variables and      }
     { properties; also, it creates objects for properties of       }
     { standard Delphi object types (e.g., TFont, TTimer,           }
     { TPicture) and for any variables marked as objects.           }
     { AutoInitialize method is generated by Component Create.      }
     AutoInitialize;

     { Code to perform other tasks when the component is created }

end; //constructor TJPMEFontComboBox.Create(AOwner: TComponent);

destructor TJPMEFontComboBox.Destroy;
begin
     { AutoDestroy, which is generated by Component Create, frees any   }
     { objects created by AutoInitialize.                               }
     AutoDestroy;

     { Here, free any other dynamic objects that the component methods  }
     { created but have not yet freed.  Also perform any other clean-up }
     { operations needed before the component is destroyed.             }

     { Last, free the component by calling the Destroy method of the    }
     { parent class.                                                    }
     inherited Destroy;
end; //destructor TJPMEFontComboBox.Destroy;

{ Draw override to display Fonts }

procedure TJPMEFontComboBox.DrawItem (Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
     {This part was taken from ESBPCS and used with permission.  The code basically provides
     a more consistant feel with ESBPCS.  I know that this is duplicated in the inherited call but
     I also will not be using that to draw the complete item.  I only use that to draw the rest
     of the combo-box.  Therefore, this is necessary.}
     if (odFocused in State) or (odSelected in State) then
     begin
          Canvas.Font.Color := ColorSet.ListHighlightFontColor;
          Canvas.Brush.Color := ColorSet.ListHighlightColor;
     end //if (odFocused in State) or (odSelected in State) then
     else
     begin
          if ReadOnly then
          begin
               Canvas.Brush.Color := ColorSet.ROColor;
               Canvas.Font.Color := Font.Color;
          end //if ReadOnly then
          else
          begin
               Canvas.Font.Color := ColorSet.ListFontColor;
               Canvas.Brush.Color := ColorSet.ListColor;
          end; //else ... if ReadOnly then
     end; //else ... if (odFocused in State) or (odSelected in State) then
     Canvas.FillRect (Rect);
     {Font Preview rectangle}
     fFontSelObj.DrawBox (Canvas, Index, Rect, State);
     inherited DrawItem (Index, Rect, State);
end; //procedure TJPMEFontComboBox.DrawItem(Index: Integer; Rect: TRect; State: TOwnerDrawState);

procedure TJPMEFontComboBox.Loaded;
begin
     inherited Loaded;

     {Populate the combobox with Fonts}
     fFontSelObj.PopulateComboBox;
     ItemIndex := Items.IndexOf (fFontSelObj.SelectedFont);
end; //procedure TJPMEFontComboBox.Loaded;

{populate the Combobox with Fonts as appopriate}

procedure TJPMEFontComboBox.PopulateComboBox;
begin
     fFontSelObj.PopulateComboBox;
     {we only will accept a value if it is listed in the combobox, otherwise, it would defeat the whole purpose}
     if Items.IndexOf (fFontSelObj.SelectedFont) <> -1 then
     begin
          ItemIndex := Items.IndexOf (fFontSelObj.SelectedFont); //select Font face in combobox
     end // if Items.IndexOf(Value) <> -1 then
     else
     begin
          {The Font selected was not on this system so we have to drop that value from the SelectedFont}
          fFontSelObj.SelectedFont := '';
          ItemIndex := -1;
     end; //else.. if Items.IndexOf(Value) <> -1 then
end; //procedure TJPMEFontComboBox.PopulateComboBox;

{property write procedure for devices}

procedure TJPMEFontComboBox.SetDevices (Value: TjpmDevices);
begin
     fFontSelObj.Devices := Value;
     {populate CombBox only after all properties are loaded}
     if (not (csReading in ComponentState)) and (not (csDesigning in ComponentState)) then
          PopulateComboBox;
end; //procedure TJPMEFontComboBox.SetDevices(Value: TjpmDevices);

{ FontTypes write procedure }

procedure TJPMEFontComboBox.SetFontTypes (Value: TjpmFontypes);
begin
     fFontSelObj.FontTypes := Value;
     if (not (csReading in ComponentState)) and (not (csDesigning in ComponentState)) then
          //only populate if we are not loading the component
          PopulateComboBox;
end; //procedure TJPMEFontComboBox.SetFontTypes(Value: TjpmFontypes);

//  if not (csReading in ComponentState) then
//    Repaint;  //force redraw when property is changed

{ SelectedFont property write procedure }

procedure TJPMEFontComboBox.SetSelectedFont (Value: string);
begin
     {if the component is in designing state or Reading, we permit the value for the moment but drop it if the EnumFontEx never returns it}
     if (csReading in ComponentState) or (csDesigning in ComponentState) then
          fFontSelObj.SelectedFont := Value //we add it for now
     else if Items.IndexOf (Value) <> -1 then
     begin
          fFontSelObj.SelectedFont := Value;
          ItemIndex := Items.IndexOf (Value); //select Font face in combobox
     end; // if Items.IndexOf(Value) <> -1 then
end; //procedure TJPMEFontComboBox.SetSelectedFont(Value: String);

function TJPMEFontComboBox.GetDevices: TjpmDevices;
begin
     Result := fFontSelObj.Devices;
end;

function TJPMEFontComboBox.GetFontSample: string;
begin
     Result := fFontSelObj.FontSample;
end;

function TJPMEFontComboBox.GetFontTypes: TjpmFontypes;
begin
     Result := fFontSelObj.FontTypes;
end;

function TJPMEFontComboBox.GetPreviewBoxWidth: Integer;
begin
     Result := fFontSelObj.ShowBoxWidth;
end;

function TJPMEFontComboBox.GetSelectedFont: string;
begin
     Result := fFontSelObj.SelectedFont;
end;

procedure TJPMEFontComboBox.SetFontSample (Value: string);
begin
     fFontSelObj.FontSample := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

procedure TJPMEFontComboBox.SetPreviewBoxWidth (Value: Integer);
begin
     fFontSelObj.ShowBoxWidth := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

procedure TJPMEFontComboBox.SetParent (AParent: TWinControl);
var
     Populate: Boolean;
begin
     Populate := (AParent <> nil) and (AParent <> Parent) and
          (not (csReading in ComponentState)) and
          (not (csLoading in ComponentState)) and
          (not (csDesigning in ComponentState));
     inherited;
     if Populate then
          PopulateComboBox;
end;

function TJPMEFontComboBox.GetFontPitches: TjpmFontPitches;
begin
     Result := fFontSelObj.FontPitches;
end;

procedure TJPMEFontComboBox.SetFontPitches (Value: TjpmFontPitches);
begin
     fFontSelObj.FontPitches := Value;
     {populate CombBox only after all properties are loaded}
     if (not (csReading in ComponentState)) and (not (csDesigning in ComponentState)) then
          PopulateComboBox;
end;

function TJPMEFontComboBox.GetShowBox: Boolean;
begin
     Result := fFontSelObj.ShowBox;
end;

function TJPMEFontComboBox.GetShowBoxBorder: Boolean;
begin
     Result := fFontSelObj.ShowBoxBorder;
end;

function TJPMEFontComboBox.GetShowStyledText: Boolean;
begin
     Result := fFontSelObj.ShowStyledText;
end;

procedure TJPMEFontComboBox.SetShowBox (const Value: Boolean);
begin
     fFontSelObj.ShowBox := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

procedure TJPMEFontComboBox.SetShowBoxBorder (const Value: Boolean);
begin
     fFontSelObj.ShowBoxBorder := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

procedure TJPMEFontComboBox.SetShowStyledText (const Value: Boolean);
begin
     fFontSelObj.ShowStyledText := Value;
     if not (csReading in ComponentState) then
          Repaint; //force redraw when property is Click
end;

end.
